import { getToken, getUser } from '../services/auth';
import instance from '../models/axios';
import { KeyedMutator } from 'swr/dist/types';

export interface ReviewProps {
  id: number,
  userId: number,
  text: string,
  averageRating: number,
  userName: string,
  mutate: KeyedMutator<any>,
}

export const Review = ({id, userId, text, averageRating, userName, mutate}: ReviewProps) => {
  const user = getUser();

  const deleteReview = async () => {
    const headers = {
      'Content-Type': 'application/json',
      'Authorization': getToken()
    };
    await instance.delete(`reviews/${id}`, { headers });
    await mutate();
    alert('Review deleted');
  };

  return (
    <div className="w-full h-40 p-2">
      <div className="border-solid border-4 border-slate-900 bg-slate-300 w-full h-full p-2 ">
        <div className="flex justify-between font-bold">
          <p>Username: {userName}</p>
          <p>Average Rating: {averageRating}</p>
          {(user?.role === 'ADMIN' || user?.sub == userId) &&
            (<button onClick={deleteReview} className="border-solid border-2 rounded-lg border-slate-900 p-1 bg-red-600">
                Delete
              </button>
            )
          }
        </div>
        <p>{text}</p>
      </div>
    </div>
  )
};

export default Review;
